@@ -11,10 +11,8 @@
 import html
 import os
 import re
-import warnings
-from collections import namedtuple
 from os import path
-from typing import Any, Dict, List, Set, Tuple
+from typing import Any, Dict, List, NamedTuple, Set, Tuple
 from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
 
 from docutils import nodes
@@ -23,7 +21,6 @@ from docutils.utils import smartquotes
 
 from sphinx import addnodes
 from sphinx.builders.html import BuildInfo, StandaloneHTMLBuilder
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import __
 from sphinx.util import logging, status_iterator
 from sphinx.util.fileutil import copy_asset_file
@@ -84,10 +81,30 @@ VECTOR_GRAPHICS_EXTENSIONS = ('.svg',)
 REFURI_RE = re.compile("([^#:]*#)(.*)")
 
 
-ManifestItem = namedtuple('ManifestItem', ['href', 'id', 'media_type'])
-Spine = namedtuple('Spine', ['idref', 'linear'])
-Guide = namedtuple('Guide', ['type', 'title', 'uri'])
-NavPoint = namedtuple('NavPoint', ['navpoint', 'playorder', 'text', 'refuri', 'children'])
+class ManifestItem(NamedTuple):
+    href: str
+    id: str
+    media_type: str
+
+
+class Spine(NamedTuple):
+    idref: str
+    linear: bool
+
+
+class Guide(NamedTuple):
+    type: str
+    title: str
+    uri: str
+
+
+class NavPoint(NamedTuple):
+    navpoint: str
+    playorder: int
+    text: str
+    refuri: str
+    children: List[Any]     # mypy does not support recursive types
+                            # https://github.com/python/mypy/issues/7069
 
 
 def sphinx_smarty_pants(t: str, language: str = 'en') -> str:
@@ -126,7 +143,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
     embedded = True
     # disable download role
     download_support = False
-    # dont' create links to original images from images
+    # don't create links to original images from images
     html_scaled_image_link = False
     # don't generate search index or include search page
     search = False
@@ -148,9 +165,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.link_suffix = '.xhtml'
         self.playorder = 0
         self.tocid = 0
-        self.id_cache = {}  # type: Dict[str, str]
+        self.id_cache: Dict[str, str] = {}
         self.use_index = self.get_builder_config('use_index', 'epub')
-        self.refnodes = []  # type: List[Dict[str, Any]]
+        self.refnodes: List[Dict[str, Any]] = []
 
     def create_build_info(self) -> BuildInfo:
         return BuildInfo(self.config, self.tags, ['html', 'epub'])
@@ -168,18 +185,6 @@ class EpubBuilder(StandaloneHTMLBuilder):
             self.id_cache[name] = id
         return id
 
-    def esc(self, name: str) -> str:
-        """Replace all characters not allowed in text an attribute values."""
-        warnings.warn(
-            '%s.esc() is deprecated. Use html.escape() instead.' % self.__class__.__name__,
-            RemovedInSphinx40Warning, stacklevel=2)
-        name = name.replace('&', '&amp;')
-        name = name.replace('<', '&lt;')
-        name = name.replace('>', '&gt;')
-        name = name.replace('"', '&quot;')
-        name = name.replace('\'', '&#39;')
-        return name
-
     def get_refnodes(self, doctree: Node, result: List[Dict[str, Any]]) -> List[Dict[str, Any]]:  # NOQA
         """Collect section titles, their depth in the toc and the refuri."""
         # XXX: is there a better way than checking the attribute
@@ -204,7 +209,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         return result
 
     def check_refnodes(self, nodes: List[Dict[str, Any]]) -> None:
-        appeared = set()  # type: Set[str]
+        appeared: Set[str] = set()
         for node in nodes:
             if node['refuri'] in appeared:
                 logger.warning(
@@ -217,14 +222,14 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 appeared.add(node['refuri'])
 
     def get_toc(self) -> None:
-        """Get the total table of contents, containing the master_doc
+        """Get the total table of contents, containing the root_doc
         and pre and post files not managed by sphinx.
         """
-        doctree = self.env.get_and_resolve_doctree(self.config.master_doc,
+        doctree = self.env.get_and_resolve_doctree(self.config.root_doc,
                                                    self, prune_toctrees=False,
                                                    includehidden=True)
         self.refnodes = self.get_refnodes(doctree, [])
-        master_dir = path.dirname(self.config.master_doc)
+        master_dir = path.dirname(self.config.root_doc)
         if master_dir:
             master_dir += '/'  # XXX or os.sep?
             for item in self.refnodes:
@@ -232,13 +237,13 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.toc_add_files(self.refnodes)
 
     def toc_add_files(self, refnodes: List[Dict[str, Any]]) -> None:
-        """Add the master_doc, pre and post files to a list of refnodes.
+        """Add the root_doc, pre and post files to a list of refnodes.
         """
         refnodes.insert(0, {
             'level': 1,
-            'refuri': html.escape(self.config.master_doc + self.out_suffix),
+            'refuri': html.escape(self.config.root_doc + self.out_suffix),
             'text': ssp(html.escape(
-                self.env.titles[self.config.master_doc].astext()))
+                self.env.titles[self.config.root_doc].astext()))
         })
         for file, text in reversed(self.config.epub_pre_files):
             refnodes.insert(0, {
@@ -265,7 +270,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         def update_node_id(node: Element) -> None:
             """Update IDs of given *node*."""
-            new_ids = []
+            new_ids: List[str] = []
             for node_id in node['ids']:
                 new_id = self.fix_fragment('', node_id)
                 if new_id not in new_ids:
@@ -283,7 +288,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         for target in tree.traverse(nodes.target):
             update_node_id(target)
 
-            next_node = target.next_node(ascend=True)  # type: Node
+            next_node: Node = target.next_node(ascend=True)
             if isinstance(next_node, nodes.Element):
                 update_node_id(next_node)
 
@@ -461,36 +466,23 @@ class EpubBuilder(StandaloneHTMLBuilder):
         addctx['doctype'] = self.doctype
         super().handle_page(pagename, addctx, templatename, outfilename, event_arg)
 
-    def build_mimetype(self, outdir: str = None, outname: str = 'mimetype') -> None:
+    def build_mimetype(self) -> None:
         """Write the metainfo file mimetype."""
-        if outdir:
-            warnings.warn('The arguments of EpubBuilder.build_mimetype() is deprecated.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-        else:
-            outdir = self.outdir
+        logger.info(__('writing mimetype file...'))
+        copy_asset_file(path.join(self.template_dir, 'mimetype'), self.outdir)
 
-        logger.info(__('writing %s file...'), outname)
-        copy_asset_file(path.join(self.template_dir, 'mimetype'),
-                        path.join(outdir, outname))
-
-    def build_container(self, outdir: str = None, outname: str = 'META-INF/container.xml') -> None:  # NOQA
+    def build_container(self, outname: str = 'META-INF/container.xml') -> None:  # NOQA
         """Write the metainfo file META-INF/container.xml."""
-        if outdir:
-            warnings.warn('The arguments of EpubBuilder.build_container() is deprecated.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-        else:
-            outdir = self.outdir
-
-        logger.info(__('writing %s file...'), outname)
-        filename = path.join(outdir, outname)
-        ensuredir(path.dirname(filename))
-        copy_asset_file(path.join(self.template_dir, 'container.xml'), filename)
+        logger.info(__('writing META-INF/container.xml file...'))
+        outdir = path.join(self.outdir, 'META-INF')
+        ensuredir(outdir)
+        copy_asset_file(path.join(self.template_dir, 'container.xml'), outdir)
 
     def content_metadata(self) -> Dict[str, Any]:
         """Create a dictionary with all metadata for the content.opf
         file properly escaped.
         """
-        metadata = {}  # type: Dict[str, Any]
+        metadata: Dict[str, Any] = {}
         metadata['title'] = html.escape(self.config.epub_title)
         metadata['author'] = html.escape(self.config.epub_author)
         metadata['uid'] = html.escape(self.config.epub_uid)
@@ -505,24 +497,18 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata['guides'] = []
         return metadata
 
-    def build_content(self, outdir: str = None, outname: str = 'content.opf') -> None:
+    def build_content(self) -> None:
         """Write the metainfo file content.opf It contains bibliographic data,
         a file list and the spine (the reading order).
         """
-        if outdir:
-            warnings.warn('The arguments of EpubBuilder.build_content() is deprecated.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-        else:
-            outdir = self.outdir
-
-        logger.info(__('writing %s file...'), outname)
+        logger.info(__('writing content.opf file...'))
         metadata = self.content_metadata()
 
         # files
-        if not outdir.endswith(os.sep):
-            outdir += os.sep
-        olen = len(outdir)
-        self.files = []  # type: List[str]
+        if not self.outdir.endswith(os.sep):
+            self.outdir += os.sep
+        olen = len(self.outdir)
+        self.files: List[str] = []
         self.ignored_files = ['.buildinfo', 'mimetype', 'content.opf',
                               'toc.ncx', 'META-INF/container.xml',
                               'Thumbs.db', 'ehthumbs.db', '.DS_Store',
@@ -530,7 +516,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
             self.config.epub_exclude_files
         if not self.use_index:
             self.ignored_files.append('genindex' + self.out_suffix)
-        for root, dirs, files in os.walk(outdir):
+        for root, dirs, files in os.walk(self.outdir):
             dirs.sort()
             for fn in sorted(files):
                 filename = path.join(root, fn)[olen:]
@@ -620,9 +606,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                                             html.escape(self.refnodes[0]['refuri'])))
 
         # write the project file
-        copy_asset_file(path.join(self.template_dir, 'content.opf_t'),
-                        path.join(outdir, outname),
-                        metadata)
+        copy_asset_file(path.join(self.template_dir, 'content.opf_t'), self.outdir, metadata)
 
     def new_navpoint(self, node: Dict[str, Any], level: int, incr: bool = True) -> NavPoint:
         """Create a new entry in the toc from the node at given level."""
@@ -639,8 +623,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
         Subelements of a node are nested inside the navpoint.  For nested nodes
         the parent node is reinserted in the subnav.
         """
-        navstack = []  # type: List[NavPoint]
-        navstack.append(NavPoint('dummy', '', '', '', []))
+        navstack: List[NavPoint] = []
+        navstack.append(NavPoint('dummy', 0, '', '', []))
         level = 0
         lastnode = None
         for node in nodes:
@@ -681,25 +665,19 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """Create a dictionary with all metadata for the toc.ncx file
         properly escaped.
         """
-        metadata = {}  # type: Dict[str, Any]
+        metadata: Dict[str, Any] = {}
         metadata['uid'] = self.config.epub_uid
         metadata['title'] = html.escape(self.config.epub_title)
         metadata['level'] = level
         metadata['navpoints'] = navpoints
         return metadata
 
-    def build_toc(self, outdir: str = None, outname: str = 'toc.ncx') -> None:
+    def build_toc(self) -> None:
         """Write the metainfo file toc.ncx."""
-        if outdir:
-            warnings.warn('The arguments of EpubBuilder.build_toc() is deprecated.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-        else:
-            outdir = self.outdir
-
-        logger.info(__('writing %s file...'), outname)
+        logger.info(__('writing toc.ncx file...'))
 
         if self.config.epub_tocscope == 'default':
-            doctree = self.env.get_and_resolve_doctree(self.config.master_doc,
+            doctree = self.env.get_and_resolve_doctree(self.config.root_doc,
                                                        self, prune_toctrees=False,
                                                        includehidden=False)
             refnodes = self.get_refnodes(doctree, [])
@@ -711,28 +689,21 @@ class EpubBuilder(StandaloneHTMLBuilder):
         navpoints = self.build_navpoints(refnodes)
         level = max(item['level'] for item in self.refnodes)
         level = min(level, self.config.epub_tocdepth)
-        copy_asset_file(path.join(self.template_dir, 'toc.ncx_t'),
-                        path.join(outdir, outname),
+        copy_asset_file(path.join(self.template_dir, 'toc.ncx_t'), self.outdir,
                         self.toc_metadata(level, navpoints))
 
-    def build_epub(self, outdir: str = None, outname: str = None) -> None:
+    def build_epub(self) -> None:
         """Write the epub file.
 
         It is a zip file with the mimetype file stored uncompressed as the first
         entry.
         """
-        if outdir:
-            warnings.warn('The arguments of EpubBuilder.build_epub() is deprecated.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-        else:
-            outdir = self.outdir
-            outname = self.config.epub_basename + '.epub'
-
+        outname = self.config.epub_basename + '.epub'
         logger.info(__('writing %s file...'), outname)
-        epub_filename = path.join(outdir, outname)
+        epub_filename = path.join(self.outdir, outname)
         with ZipFile(epub_filename, 'w', ZIP_DEFLATED) as epub:
-            epub.write(path.join(outdir, 'mimetype'), 'mimetype', ZIP_STORED)
+            epub.write(path.join(self.outdir, 'mimetype'), 'mimetype', ZIP_STORED)
             for filename in ['META-INF/container.xml', 'content.opf', 'toc.ncx']:
-                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)
+                epub.write(path.join(self.outdir, filename), filename, ZIP_DEFLATED)
             for filename in self.files:
-                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)
+                epub.write(path.join(self.outdir, filename), filename, ZIP_DEFLATED)
