@@ -2,7 +2,7 @@
     sphinx.util.typing
     ~~~~~~~~~~~~~~~~~~
 
-    The composit types for Sphinx.
+    The composite types for Sphinx.
 
     :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
@@ -10,18 +10,22 @@
 
 import sys
 import typing
-from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, TypeVar, Union
+from struct import Struct
+from types import TracebackType
+from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Type, TypeVar, Union
 
 from docutils import nodes
 from docutils.parsers.rst.states import Inliner
 
+from sphinx.deprecation import RemovedInSphinx60Warning, deprecated_alias
+
 if sys.version_info > (3, 7):
     from typing import ForwardRef
 else:
     from typing import _ForwardRef  # type: ignore
 
     class ForwardRef:
-        """A pseudo ForwardRef class for py35 and py36."""
+        """A pseudo ForwardRef class for py36."""
         def __init__(self, arg: Any, is_argument: bool = True) -> None:
             self.arg = arg
 
@@ -29,13 +33,22 @@ else:
             ref = _ForwardRef(self.arg)
             return ref._eval_type(globalns, localns)
 
+try:
+    from types import UnionType  # type: ignore  # python 3.10 or above
+except ImportError:
+    UnionType = None
+
 if False:
     # For type annotation
     from typing import Type  # NOQA # for python3.5.1
 
 
-# An entry of Directive.option_spec
-DirectiveOption = Callable[[str], Any]
+# builtin classes that have incorrect __module__
+INVALID_BUILTIN_CLASSES = {
+    Struct: 'struct.Struct',  # Before Python 3.9
+    TracebackType: 'types.TracebackType',
+}
+
 
 # Text like nodes which are initialized with text and rawsource
 TextlikeNode = Union[nodes.Text, nodes.TextElement]
@@ -50,6 +63,9 @@ PathMatcher = Callable[[str], bool]
 RoleFunction = Callable[[str, str, str, int, Inliner, Dict[str, Any], List[str]],
                         Tuple[List[nodes.Node], List[nodes.system_message]]]
 
+# A option spec for directive
+OptionSpec = Dict[str, Callable[[str], Any]]
+
 # title getter functions for enumerable nodes (see sphinx.domains.std)
 TitleGetter = Callable[[nodes.Node], str]
 
@@ -70,14 +86,16 @@ def get_type_hints(obj: Any, globalns: Dict = None, localns: Dict = None) -> Dic
     except NameError:
         # Failed to evaluate ForwardRef (maybe TYPE_CHECKING)
         return safe_getattr(obj, '__annotations__', {})
+    except AttributeError:
+        # Failed to evaluate ForwardRef (maybe not runtime checkable)
+        return safe_getattr(obj, '__annotations__', {})
     except TypeError:
-        return {}
+        # Invalid object is given. But try to get __annotations__ as a fallback for
+        # the code using type union operator (PEP 604) in python 3.9 or below.
+        return safe_getattr(obj, '__annotations__', {})
     except KeyError:
         # a broken class found (refs: https://github.com/sphinx-doc/sphinx/issues/8084)
         return {}
-    except AttributeError:
-        # AttributeError is raised on 3.5.2 (fixed by 3.5.3)
-        return {}
 
 
 def is_system_TypeVar(typ: Any) -> bool:
@@ -86,26 +104,48 @@ def is_system_TypeVar(typ: Any) -> bool:
     return modname == 'typing' and isinstance(typ, TypeVar)
 
 
-def restify(cls: Optional["Type"]) -> str:
+def restify(cls: Optional[Type]) -> str:
     """Convert python class to a reST reference."""
     from sphinx.util import inspect  # lazy loading
 
-    if cls is None or cls is NoneType:
-        return ':obj:`None`'
-    elif cls is Ellipsis:
-        return '...'
-    elif inspect.isNewType(cls):
-        return ':class:`%s`' % cls.__name__
-    elif cls.__module__ in ('__builtin__', 'builtins'):
-        return ':class:`%s`' % cls.__name__
-    else:
-        if sys.version_info >= (3, 7):  # py37+
-            return _restify_py37(cls)
+    try:
+        if cls is None or cls is NoneType:
+            return ':obj:`None`'
+        elif cls is Ellipsis:
+            return '...'
+        elif cls in INVALID_BUILTIN_CLASSES:
+            return ':class:`%s`' % INVALID_BUILTIN_CLASSES[cls]
+        elif inspect.isNewType(cls):
+            if sys.version_info > (3, 10):
+                # newtypes have correct module info since Python 3.10+
+                print(cls, type(cls), dir(cls))
+                return ':class:`%s.%s`' % (cls.__module__, cls.__name__)
+            else:
+                return ':class:`%s`' % cls.__name__
+        elif UnionType and isinstance(cls, UnionType):
+            if len(cls.__args__) > 1 and None in cls.__args__:
+                args = ' | '.join(restify(a) for a in cls.__args__ if a)
+                return 'Optional[%s]' % args
+            else:
+                return ' | '.join(restify(a) for a in cls.__args__)
+        elif cls.__module__ in ('__builtin__', 'builtins'):
+            if hasattr(cls, '__args__'):
+                return ':class:`%s`\\ [%s]' % (
+                    cls.__name__,
+                    ', '.join(restify(arg) for arg in cls.__args__),
+                )
+            else:
+                return ':class:`%s`' % cls.__name__
         else:
-            return _restify_py36(cls)
+            if sys.version_info >= (3, 7):  # py37+
+                return _restify_py37(cls)
+            else:
+                return _restify_py36(cls)
+    except (AttributeError, TypeError):
+        return repr(cls)
 
 
-def _restify_py37(cls: Optional["Type"]) -> str:
+def _restify_py37(cls: Optional[Type]) -> str:
     """Convert python class to a reST reference."""
     from sphinx.util import inspect  # lazy loading
 
@@ -115,21 +155,24 @@ def _restify_py37(cls: Optional["Type"]) -> str:
         if len(cls.__args__) > 1 and cls.__args__[-1] is NoneType:
             if len(cls.__args__) > 2:
                 args = ', '.join(restify(a) for a in cls.__args__[:-1])
-                return ':obj:`Optional`\\ [:obj:`Union`\\ [%s]]' % args
+                return ':obj:`~typing.Optional`\\ [:obj:`~typing.Union`\\ [%s]]' % args
             else:
-                return ':obj:`Optional`\\ [%s]' % restify(cls.__args__[0])
+                return ':obj:`~typing.Optional`\\ [%s]' % restify(cls.__args__[0])
         else:
             args = ', '.join(restify(a) for a in cls.__args__)
-            return ':obj:`Union`\\ [%s]' % args
+            return ':obj:`~typing.Union`\\ [%s]' % args
     elif inspect.isgenericalias(cls):
-        if getattr(cls, '_name', None):
+        if isinstance(cls.__origin__, typing._SpecialForm):
+            text = restify(cls.__origin__)  # type: ignore
+        elif getattr(cls, '_name', None):
             if cls.__module__ == 'typing':
-                text = ':class:`%s`' % cls._name
+                text = ':class:`~%s.%s`' % (cls.__module__, cls._name)
             else:
                 text = ':class:`%s.%s`' % (cls.__module__, cls._name)
         else:
             text = restify(cls.__origin__)
 
+        origin = getattr(cls, '__origin__', None)
         if not hasattr(cls, '__args__'):
             pass
         elif all(is_system_TypeVar(a) for a in cls.__args__):
@@ -138,29 +181,30 @@ def _restify_py37(cls: Optional["Type"]) -> str:
         elif cls.__module__ == 'typing' and cls._name == 'Callable':
             args = ', '.join(restify(a) for a in cls.__args__[:-1])
             text += r"\ [[%s], %s]" % (args, restify(cls.__args__[-1]))
+        elif cls.__module__ == 'typing' and getattr(origin, '_name', None) == 'Literal':
+            text += r"\ [%s]" % ', '.join(repr(a) for a in cls.__args__)
         elif cls.__args__:
             text += r"\ [%s]" % ", ".join(restify(a) for a in cls.__args__)
 
         return text
+    elif isinstance(cls, typing._SpecialForm):
+        return ':obj:`~%s.%s`' % (cls.__module__, cls._name)
     elif hasattr(cls, '__qualname__'):
         if cls.__module__ == 'typing':
-            return ':class:`%s`' % cls.__qualname__
+            return ':class:`~%s.%s`' % (cls.__module__, cls.__qualname__)
         else:
             return ':class:`%s.%s`' % (cls.__module__, cls.__qualname__)
-    elif hasattr(cls, '_name'):
-        # SpecialForm
-        if cls.__module__ == 'typing':
-            return ':obj:`%s`' % cls._name
-        else:
-            return ':obj:`%s.%s`' % (cls.__module__, cls._name)
     elif isinstance(cls, ForwardRef):
         return ':class:`%s`' % cls.__forward_arg__
     else:
         # not a class (ex. TypeVar)
-        return ':obj:`%s.%s`' % (cls.__module__, cls.__name__)
+        if cls.__module__ == 'typing':
+            return ':obj:`~%s.%s`' % (cls.__module__, cls.__name__)
+        else:
+            return ':obj:`%s.%s`' % (cls.__module__, cls.__name__)
 
 
-def _restify_py36(cls: Optional["Type"]) -> str:
+def _restify_py36(cls: Optional[Type]) -> str:
     module = getattr(cls, '__module__', None)
     if module == 'typing':
         if getattr(cls, '_name', None):
@@ -179,104 +223,75 @@ def _restify_py36(cls: Optional["Type"]) -> str:
         qualname = repr(cls)
 
     if (isinstance(cls, typing.TupleMeta) and  # type: ignore
-            not hasattr(cls, '__tuple_params__')):  # for Python 3.6
+            not hasattr(cls, '__tuple_params__')):
+        if module == 'typing':
+            reftext = ':class:`~typing.%s`' % qualname
+        else:
+            reftext = ':class:`%s`' % qualname
+
         params = cls.__args__
         if params:
             param_str = ', '.join(restify(p) for p in params)
-            return ':class:`%s`\\ [%s]' % (qualname, param_str)
+            return reftext + '\\ [%s]' % param_str
         else:
-            return ':class:`%s`' % qualname
+            return reftext
     elif isinstance(cls, typing.GenericMeta):
-        params = None
-        if hasattr(cls, '__args__'):
-            # for Python 3.5.2+
-            if cls.__args__ is None or len(cls.__args__) <= 2:  # type: ignore  # NOQA
-                params = cls.__args__  # type: ignore
-            elif cls.__origin__ == Generator:  # type: ignore
-                params = cls.__args__  # type: ignore
-            else:  # typing.Callable
-                args = ', '.join(restify(arg) for arg in cls.__args__[:-1])  # type: ignore
-                result = restify(cls.__args__[-1])  # type: ignore
-                return ':class:`%s`\\ [[%s], %s]' % (qualname, args, result)
-        elif hasattr(cls, '__parameters__'):
-            # for Python 3.5.0 and 3.5.1
-            params = cls.__parameters__  # type: ignore
+        if module == 'typing':
+            reftext = ':class:`~typing.%s`' % qualname
+        else:
+            reftext = ':class:`%s`' % qualname
+
+        if cls.__args__ is None or len(cls.__args__) <= 2:
+            params = cls.__args__
+        elif cls.__origin__ == Generator:
+            params = cls.__args__
+        else:  # typing.Callable
+            args = ', '.join(restify(arg) for arg in cls.__args__[:-1])
+            result = restify(cls.__args__[-1])
+            return reftext + '\\ [[%s], %s]' % (args, result)
 
         if params:
             param_str = ', '.join(restify(p) for p in params)
-            return ':class:`%s`\\ [%s]' % (qualname, param_str)
+            return reftext + '\\ [%s]' % (param_str)
         else:
-            return ':class:`%s`' % qualname
-    elif (hasattr(typing, 'UnionMeta') and
-            isinstance(cls, typing.UnionMeta) and  # type: ignore
-            hasattr(cls, '__union_params__')):  # for Python 3.5
-        params = cls.__union_params__
-        if params is not None:
-            if len(params) == 2 and params[1] is NoneType:
-                return ':obj:`Optional`\\ [%s]' % restify(params[0])
-            else:
-                param_str = ', '.join(restify(p) for p in params)
-                return ':obj:`%s`\\ [%s]' % (qualname, param_str)
-        else:
-            return ':obj:`%s`' % qualname
+            return reftext
     elif (hasattr(cls, '__origin__') and
-          cls.__origin__ is typing.Union):  # for Python 3.5.2+
+          cls.__origin__ is typing.Union):
         params = cls.__args__
         if params is not None:
             if len(params) > 1 and params[-1] is NoneType:
                 if len(params) > 2:
                     param_str = ", ".join(restify(p) for p in params[:-1])
-                    return ':obj:`Optional`\\ [:obj:`Union`\\ [%s]]' % param_str
+                    return (':obj:`~typing.Optional`\\ '
+                            '[:obj:`~typing.Union`\\ [%s]]' % param_str)
                 else:
-                    return ':obj:`Optional`\\ [%s]' % restify(params[0])
+                    return ':obj:`~typing.Optional`\\ [%s]' % restify(params[0])
             else:
                 param_str = ', '.join(restify(p) for p in params)
-                return ':obj:`Union`\\ [%s]' % param_str
+                return ':obj:`~typing.Union`\\ [%s]' % param_str
         else:
             return ':obj:`Union`'
-    elif (isinstance(cls, typing.CallableMeta) and  # type: ignore
-          getattr(cls, '__args__', None) is not None and
-          hasattr(cls, '__result__')):  # for Python 3.5
-        # Skipped in the case of plain typing.Callable
-        args = cls.__args__
-        if args is None:
-            return qualname
-        elif args is Ellipsis:
-            args_str = '...'
-        else:
-            formatted_args = (restify(a) for a in args)  # type: ignore
-            args_str = '[%s]' % ', '.join(formatted_args)
-
-        return ':class:`%s`\\ [%s, %s]' % (qualname, args_str, stringify(cls.__result__))
-    elif (isinstance(cls, typing.TupleMeta) and  # type: ignore
-          hasattr(cls, '__tuple_params__') and
-          hasattr(cls, '__tuple_use_ellipsis__')):  # for Python 3.5
-        params = cls.__tuple_params__
-        if params is not None:
-            param_strings = [restify(p) for p in params]
-            if cls.__tuple_use_ellipsis__:
-                param_strings.append('...')
-            return ':class:`%s`\\ [%s]' % (qualname, ', '.join(param_strings))
-        else:
-            return ':class:`%s`' % qualname
     elif hasattr(cls, '__qualname__'):
         if cls.__module__ == 'typing':
-            return ':class:`%s`' % cls.__qualname__
+            return ':class:`~%s.%s`' % (cls.__module__, cls.__qualname__)
         else:
             return ':class:`%s.%s`' % (cls.__module__, cls.__qualname__)
     elif hasattr(cls, '_name'):
         # SpecialForm
         if cls.__module__ == 'typing':
-            return ':obj:`%s`' % cls._name
+            return ':obj:`~%s.%s`' % (cls.__module__, cls._name)
         else:
             return ':obj:`%s.%s`' % (cls.__module__, cls._name)
     elif hasattr(cls, '__name__'):
         # not a class (ex. TypeVar)
-        return ':obj:`%s.%s`' % (cls.__module__, cls.__name__)
+        if cls.__module__ == 'typing':
+            return ':obj:`~%s.%s`' % (cls.__module__, cls.__name__)
+        else:
+            return ':obj:`%s.%s`' % (cls.__module__, cls.__name__)
     else:
         # others (ex. Any)
         if cls.__module__ == 'typing':
-            return ':obj:`%s`' % qualname
+            return ':obj:`~%s.%s`' % (cls.__module__, qualname)
         else:
             return ':obj:`%s.%s`' % (cls.__module__, qualname)
 
@@ -292,17 +307,30 @@ def stringify(annotation: Any) -> str:
         else:
             return annotation
     elif isinstance(annotation, TypeVar):
-        return annotation.__name__
+        if annotation.__module__ == 'typing':
+            return annotation.__name__
+        else:
+            return '.'.join([annotation.__module__, annotation.__name__])
     elif inspect.isNewType(annotation):
-        # Could not get the module where it defiend
-        return annotation.__name__
+        if sys.version_info > (3, 10):
+            # newtypes have correct module info since Python 3.10+
+            return '%s.%s' % (annotation.__module__, annotation.__name__)
+        else:
+            return annotation.__name__
     elif not annotation:
         return repr(annotation)
     elif annotation is NoneType:
         return 'None'
+    elif annotation in INVALID_BUILTIN_CLASSES:
+        return INVALID_BUILTIN_CLASSES[annotation]
+    elif str(annotation).startswith('typing.Annotated'):  # for py310+
+        pass
     elif (getattr(annotation, '__module__', None) == 'builtins' and
-          hasattr(annotation, '__qualname__')):
-        return annotation.__qualname__
+          getattr(annotation, '__qualname__', None)):
+        if hasattr(annotation, '__args__'):  # PEP 585 generic
+            return repr(annotation)
+        else:
+            return annotation.__qualname__
     elif annotation is Ellipsis:
         return '...'
 
@@ -329,6 +357,8 @@ def _stringify_py37(annotation: Any) -> str:
     elif hasattr(annotation, '__origin__'):
         # instantiated generic provided by a user
         qualname = stringify(annotation.__origin__)
+    elif UnionType and isinstance(annotation, UnionType):  # types.Union (for py3.10+)
+        qualname = 'types.Union'
     else:
         # we weren't able to extract the base type, appending arguments would
         # only make them appear twice
@@ -338,7 +368,7 @@ def _stringify_py37(annotation: Any) -> str:
         if not isinstance(annotation.__args__, (list, tuple)):
             # broken __args__ found
             pass
-        elif qualname == 'Union':
+        elif qualname in ('Optional', 'Union'):
             if len(annotation.__args__) > 1 and annotation.__args__[-1] is NoneType:
                 if len(annotation.__args__) > 2:
                     args = ', '.join(stringify(a) for a in annotation.__args__[:-1])
@@ -348,10 +378,19 @@ def _stringify_py37(annotation: Any) -> str:
             else:
                 args = ', '.join(stringify(a) for a in annotation.__args__)
                 return 'Union[%s]' % args
+        elif qualname == 'types.Union':
+            if len(annotation.__args__) > 1 and None in annotation.__args__:
+                args = ' | '.join(stringify(a) for a in annotation.__args__ if a)
+                return 'Optional[%s]' % args
+            else:
+                return ' | '.join(stringify(a) for a in annotation.__args__)
         elif qualname == 'Callable':
             args = ', '.join(stringify(a) for a in annotation.__args__[:-1])
             returns = stringify(annotation.__args__[-1])
             return '%s[[%s], %s]' % (qualname, args, returns)
+        elif qualname == 'Literal':
+            args = ', '.join(repr(a) for a in annotation.__args__)
+            return '%s[%s]' % (qualname, args)
         elif str(annotation).startswith('typing.Annotated'):  # for py39+
             return stringify(annotation.__args__[0])
         elif all(is_system_TypeVar(a) for a in annotation.__args__):
@@ -365,7 +404,7 @@ def _stringify_py37(annotation: Any) -> str:
 
 
 def _stringify_py36(annotation: Any) -> str:
-    """stringify() for py35 and py36."""
+    """stringify() for py36."""
     module = getattr(annotation, '__module__', None)
     if module == 'typing':
         if getattr(annotation, '_name', None):
@@ -393,35 +432,20 @@ def _stringify_py36(annotation: Any) -> str:
             return qualname
     elif isinstance(annotation, typing.GenericMeta):
         params = None
-        if hasattr(annotation, '__args__'):
-            # for Python 3.5.2+
-            if annotation.__args__ is None or len(annotation.__args__) <= 2:  # type: ignore  # NOQA
-                params = annotation.__args__  # type: ignore
-            elif annotation.__origin__ == Generator:  # type: ignore
-                params = annotation.__args__  # type: ignore
-            else:  # typing.Callable
-                args = ', '.join(stringify(arg) for arg
-                                 in annotation.__args__[:-1])  # type: ignore
-                result = stringify(annotation.__args__[-1])  # type: ignore
-                return '%s[[%s], %s]' % (qualname, args, result)
-        elif hasattr(annotation, '__parameters__'):
-            # for Python 3.5.0 and 3.5.1
-            params = annotation.__parameters__  # type: ignore
+        if annotation.__args__ is None or len(annotation.__args__) <= 2:  # type: ignore  # NOQA
+            params = annotation.__args__  # type: ignore
+        elif annotation.__origin__ == Generator:  # type: ignore
+            params = annotation.__args__  # type: ignore
+        else:  # typing.Callable
+            args = ', '.join(stringify(arg) for arg
+                             in annotation.__args__[:-1])  # type: ignore
+            result = stringify(annotation.__args__[-1])  # type: ignore
+            return '%s[[%s], %s]' % (qualname, args, result)
         if params is not None:
             param_str = ', '.join(stringify(p) for p in params)
             return '%s[%s]' % (qualname, param_str)
-    elif (hasattr(typing, 'UnionMeta') and
-          isinstance(annotation, typing.UnionMeta) and  # type: ignore
-          hasattr(annotation, '__union_params__')):  # for Python 3.5
-        params = annotation.__union_params__
-        if params is not None:
-            if len(params) == 2 and params[1] is NoneType:
-                return 'Optional[%s]' % stringify(params[0])
-            else:
-                param_str = ', '.join(stringify(p) for p in params)
-                return '%s[%s]' % (qualname, param_str)
     elif (hasattr(annotation, '__origin__') and
-          annotation.__origin__ is typing.Union):  # for Python 3.5.2+
+          annotation.__origin__ is typing.Union):
         params = annotation.__args__
         if params is not None:
             if len(params) > 1 and params[-1] is NoneType:
@@ -433,30 +457,12 @@ def _stringify_py36(annotation: Any) -> str:
             else:
                 param_str = ', '.join(stringify(p) for p in params)
                 return 'Union[%s]' % param_str
-    elif (isinstance(annotation, typing.CallableMeta) and  # type: ignore
-          getattr(annotation, '__args__', None) is not None and
-          hasattr(annotation, '__result__')):  # for Python 3.5
-        # Skipped in the case of plain typing.Callable
-        args = annotation.__args__
-        if args is None:
-            return qualname
-        elif args is Ellipsis:
-            args_str = '...'
-        else:
-            formatted_args = (stringify(a) for a in args)
-            args_str = '[%s]' % ', '.join(formatted_args)
-        return '%s[%s, %s]' % (qualname,
-                               args_str,
-                               stringify(annotation.__result__))
-    elif (isinstance(annotation, typing.TupleMeta) and  # type: ignore
-          hasattr(annotation, '__tuple_params__') and
-          hasattr(annotation, '__tuple_use_ellipsis__')):  # for Python 3.5
-        params = annotation.__tuple_params__
-        if params is not None:
-            param_strings = [stringify(p) for p in params]
-            if annotation.__tuple_use_ellipsis__:
-                param_strings.append('...')
-            return '%s[%s]' % (qualname,
-                               ', '.join(param_strings))
 
     return qualname
+
+
+deprecated_alias('sphinx.util.typing',
+                 {
+                     'DirectiveOption': Callable[[str], Any],
+                 },
+                 RemovedInSphinx60Warning)
