@@ -12,7 +12,8 @@ import os
 import re
 import textwrap
 from itertools import chain, groupby
-from typing import Any, Dict, Generator, Iterable, List, Optional, Set, Tuple, Union, cast
+from typing import (TYPE_CHECKING, Any, Dict, Generator, Iterable, List, Optional, Set, Tuple,
+                    Union, cast)
 
 from docutils import nodes, writers
 from docutils.nodes import Element, Node, Text
@@ -22,22 +23,21 @@ from sphinx import addnodes
 from sphinx.locale import _, admonitionlabels
 from sphinx.util.docutils import SphinxTranslator
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.builders.text import TextBuilder
 
 
 class Cell:
     """Represents a cell in a table.
-    It can span on multiple columns or on multiple lines.
+    It can span multiple columns or multiple lines.
     """
     def __init__(self, text: str = "", rowspan: int = 1, colspan: int = 1) -> None:
         self.text = text
-        self.wrapped = []  # type: List[str]
+        self.wrapped: List[str] = []
         self.rowspan = rowspan
         self.colspan = colspan
-        self.col = None  # type: Optional[int]
-        self.row = None  # type: Optional[int]
+        self.col: Optional[int] = None
+        self.row: Optional[int] = None
 
     def __repr__(self) -> str:
         return "<Cell {!r} {}v{}/{}>{}>".format(
@@ -52,7 +52,7 @@ class Cell:
 
 
 class Table:
-    """Represents a table, handling cells that can span on multiple lines
+    """Represents a table, handling cells that can span multiple lines
     or rows, like::
 
        +-----------+-----+
@@ -63,22 +63,22 @@ class Table:
        | DDD | CCC       |
        +-----+-----------+
 
-    This class can be used in two ways:
+    This class can be used in two ways, either:
 
-    - Either with absolute positions: call ``table[line, col] = Cell(...)``,
-      this overwrite an existing cell if any.
+    - With absolute positions: call ``table[line, col] = Cell(...)``,
+      this overwrites any existing cell(s) at these positions.
 
-    - Either with relative positions: call the ``add_row()`` and
+    - With relative positions: call the ``add_row()`` and
       ``add_cell(Cell(...))`` as needed.
 
-    Cell spanning on multiple rows or multiple columns (having a
+    Cells spanning multiple rows or multiple columns (having a
     colspan or rowspan greater than one) are automatically referenced
-    by all the table cells they covers. This is a useful
-    representation as we can simply check ``if self[x, y] is self[x,
-    y+1]`` to recognize a rowspan.
+    by all the table cells they cover. This is a useful
+    representation as we can simply check
+    ``if self[x, y] is self[x, y+1]`` to recognize a rowspan.
 
     Colwidth is not automatically computed, it has to be given, either
-    at construction time, either during the table construction.
+    at construction time, or during the table construction.
 
     Example usage::
 
@@ -98,10 +98,9 @@ class Table:
 
     """
     def __init__(self, colwidth: List[int] = None) -> None:
-        self.lines = []  # type: List[List[Cell]]
+        self.lines: List[List[Cell]] = []
         self.separator = 0
-        self.colwidth = (colwidth if colwidth is not None
-                         else [])  # type: List[int]
+        self.colwidth: List[int] = (colwidth if colwidth is not None else [])
         self.current_line = 0
         self.current_col = 0
 
@@ -113,14 +112,13 @@ class Table:
         self.current_col = 0
 
     def set_separator(self) -> None:
-        """Sets the separator below the current line.
-        """
+        """Sets the separator below the current line."""
         self.separator = len(self.lines)
 
     def add_cell(self, cell: Cell) -> None:
         """Add a cell to the current line, to use with ``add_row()``.  To add
-        a cell spanning on multiple lines or rows, simply set the
-        ``cell.colspan`` or ``cell.rowspan`` BEFORE inserting it to
+        a cell spanning multiple lines or rows, simply set the
+        ``cell.colspan`` or ``cell.rowspan`` BEFORE inserting it into
         the table.
         """
         while self[self.current_line, self.current_col]:
@@ -159,7 +157,7 @@ class Table:
     def cell_width(self, cell: Cell, source: List[int]) -> int:
         """Give the cell width, according to the given source (either
         ``self.colwidth`` or ``self.measured_widths``).
-        This take into account cells spanning on multiple columns.
+        This takes into account cells spanning multiple columns.
         """
         width = 0
         for i in range(self[cell.row, cell.col].colspan):
@@ -168,7 +166,7 @@ class Table:
 
     @property
     def cells(self) -> Generator[Cell, None, None]:
-        seen = set()  # type: Set[Cell]
+        seen: Set[Cell] = set()
         for lineno, line in enumerate(self.lines):
             for colno, cell in enumerate(line):
                 if cell and cell not in seen:
@@ -189,7 +187,7 @@ class Table:
                 self.measured_widths[col] = max(self.measured_widths[col], width)
 
     def physical_lines_for_line(self, line: List[Cell]) -> int:
-        """From a given line, compute the number of physical lines it spans
+        """For a given line, compute the number of physical lines it spans
         due to text wrapping.
         """
         physical_lines = 1
@@ -205,7 +203,7 @@ class Table:
             """Called on the line *before* lineno.
             Called with no *lineno* for the last sep.
             """
-            out = []  # type: List[str]
+            out: List[str] = []
             for colno, width in enumerate(self.measured_widths):
                 if (
                     lineno is not None and
@@ -267,7 +265,7 @@ class TextWrapper(textwrap.TextWrapper):
         The original _wrap_chunks uses len() to calculate width.
         This method respects wide/fullwidth characters for width adjustment.
         """
-        lines = []  # type: List[str]
+        lines: List[str] = []
         if self.width <= 0:
             raise ValueError("invalid width %r (must be > 0)" % self.width)
 
@@ -324,11 +322,11 @@ class TextWrapper(textwrap.TextWrapper):
         """_split(text : string) -> [string]
 
         Override original method that only split by 'wordsep_re'.
-        This '_split' split wide-characters into chunk by one character.
+        This '_split' splits wide-characters into chunks by one character.
         """
         def split(t: str) -> List[str]:
             return super(TextWrapper, self)._split(t)
-        chunks = []  # type: List[str]
+        chunks: List[str] = []
         for chunk in split(text):
             for w, g in groupby(chunk, column_width):
                 if w == 1:
@@ -367,9 +365,9 @@ def my_wrap(text: str, width: int = MAXWIDTH, **kwargs: Any) -> List[str]:
 class TextWriter(writers.Writer):
     supported = ('text',)
     settings_spec = ('No options here.', '', ())
-    settings_defaults = {}  # type: Dict
+    settings_defaults: Dict = {}
 
-    output = None  # type: str
+    output: str = None
 
     def __init__(self, builder: "TextBuilder") -> None:
         super().__init__()
@@ -382,7 +380,7 @@ class TextWriter(writers.Writer):
 
 
 class TextTranslator(SphinxTranslator):
-    builder = None  # type: TextBuilder
+    builder: "TextBuilder" = None
 
     def __init__(self, document: nodes.document, builder: "TextBuilder") -> None:
         super().__init__(document, builder)
@@ -397,12 +395,12 @@ class TextTranslator(SphinxTranslator):
         self.sectionchars = self.config.text_sectionchars
         self.add_secnumbers = self.config.text_add_secnumbers
         self.secnumber_suffix = self.config.text_secnumber_suffix
-        self.states = [[]]      # type: List[List[Tuple[int, Union[str, List[str]]]]]
+        self.states: List[List[Tuple[int, Union[str, List[str]]]]] = [[]]
         self.stateindent = [0]
-        self.list_counter = []  # type: List[int]
+        self.list_counter: List[int] = []
         self.sectionlevel = 0
         self.lineblocklevel = 0
-        self.table = None       # type: Table
+        self.table: Table = None
 
     def add_text(self, text: str) -> None:
         self.states[-1].append((-1, text))
@@ -415,8 +413,8 @@ class TextTranslator(SphinxTranslator):
         content = self.states.pop()
         maxindent = sum(self.stateindent)
         indent = self.stateindent.pop()
-        result = []     # type: List[Tuple[int, List[str]]]
-        toformat = []   # type: List[str]
+        result: List[Tuple[int, List[str]]] = []
+        toformat: List[str] = []
 
         def do_format() -> None:
             if not toformat:
@@ -537,6 +535,13 @@ class TextTranslator(SphinxTranslator):
     def depart_attribution(self, node: Element) -> None:
         pass
 
+    #############################################################
+    # Domain-specific object descriptions
+    #############################################################
+
+    # Top-level nodes
+    #################
+
     def visit_desc(self, node: Element) -> None:
         pass
 
@@ -556,6 +561,22 @@ class TextTranslator(SphinxTranslator):
     def depart_desc_signature_line(self, node: Element) -> None:
         self.add_text('\n')
 
+    def visit_desc_content(self, node: Element) -> None:
+        self.new_state()
+        self.add_text(self.nl)
+
+    def depart_desc_content(self, node: Element) -> None:
+        self.end_state()
+
+    def visit_desc_inline(self, node: Element) -> None:
+        pass
+
+    def depart_desc_inline(self, node: Element) -> None:
+        pass
+
+    # Nodes for high-level structure in signatures
+    ##############################################
+
     def visit_desc_name(self, node: Element) -> None:
         pass
 
@@ -607,12 +628,7 @@ class TextTranslator(SphinxTranslator):
     def depart_desc_annotation(self, node: Element) -> None:
         pass
 
-    def visit_desc_content(self, node: Element) -> None:
-        self.new_state()
-        self.add_text(self.nl)
-
-    def depart_desc_content(self, node: Element) -> None:
-        self.end_state()
+    ##############################################
 
     def visit_figure(self, node: Element) -> None:
         self.new_state()
