@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     test_api_translator
     ~~~~~~~~~~~~~~~~~~~
 
     Test the Sphinx API for translator.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,6 +12,8 @@ import sys
 
 import pytest
 
+from sphinx.util import docutils
+
 
 @pytest.fixture(scope='module', autouse=True)
 def setup_module(rootdir):
@@ -27,7 +28,10 @@ def test_html_translator(app, status, warning):
     # no set_translator()
     translator_class = app.builder.get_translator_class()
     assert translator_class
-    assert translator_class.__name__ == 'HTMLTranslator'
+    if docutils.__version_info__ < (0, 13):
+        assert translator_class.__name__ == 'HTMLTranslator'
+    else:
+        assert translator_class.__name__ == 'HTML5Translator'
 
 
 @pytest.mark.sphinx('html', testroot='api-set-translator')
