@@ -12,6 +12,7 @@
 import os
 import re
 
+import pygments
 import pytest
 from babel.messages import mofile, pofile
 from babel.messages.catalog import Catalog
@@ -30,6 +31,8 @@ sphinx_intl = pytest.mark.sphinx(
     },
 )
 
+pygments_version = tuple(int(v) for v in pygments.__version__.split('.'))
+
 
 def read_po(pathname):
     with pathname.open() as f:
@@ -336,9 +339,9 @@ def test_text_figure_captions(app):
               "14.2. IMAGE URL AND ALT\n"
               "=======================\n"
               "\n"
-              "[image: i18n][image]\n"
+              "[image: I18N -> IMG][image]\n"
               "\n"
-              "   [image: img][image]\n"
+              "   [image: IMG -> I18N][image]\n"
               "\n"
               "\n"
               "14.3. IMAGE ON SUBSTITUTION\n"
@@ -410,7 +413,7 @@ def test_text_admonitions(app):
     app.build()
     # --- admonitions
     # #1206: gettext did not translate admonition directive's title
-    # seealso: http://docutils.sourceforge.net/docs/ref/rst/directives.html#admonitions
+    # seealso: https://docutils.sourceforge.io/docs/ref/rst/directives.html#admonitions
     result = (app.outdir / 'admonitions.txt').read_text()
     directives = (
         "attention", "caution", "danger", "error", "hint",
@@ -622,7 +625,7 @@ def test_html_meta(app):
     assert expected_expr in result
     expected_expr = '<meta content="I18N, SPHINX, MARKUP" name="keywords" />'
     assert expected_expr in result
-    expected_expr = '<p class="caption"><span class="caption-text">HIDDEN TOC</span></p>'
+    expected_expr = '<p class="caption" role="heading"><span class="caption-text">HIDDEN TOC</span></p>'
     assert expected_expr in result
 
 
@@ -1060,8 +1063,13 @@ def test_additional_targets_should_not_be_translated(app):
     assert_count(expected_expr, result, 1)
 
     # C code block with lang should not be translated but be *C* highlighted
-    expected_expr = ("""<span class="cp">#include</span> """
-                     """<span class="cpf">&lt;stdio.h&gt;</span>""")
+    if pygments_version < (2, 10, 0):
+        expected_expr = ("""<span class="cp">#include</span> """
+                         """<span class="cpf">&lt;stdio.h&gt;</span>""")
+    else:
+        expected_expr = ("""<span class="cp">#include</span>"""
+                         """<span class="w"> </span>"""
+                         """<span class="cpf">&lt;stdio.h&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
     # literal block in list item should not be translated
@@ -1095,12 +1103,12 @@ def test_additional_targets_should_not_be_translated(app):
 
     result = (app.outdir / 'figure.html').read_text()
 
-    # alt and src for image block should not be translated
-    expected_expr = """<img alt="i18n" src="_images/i18n.png" />"""
+    # src for image block should not be translated (alt is translated)
+    expected_expr = """<img alt="I18N -&gt; IMG" src="_images/i18n.png" />"""
     assert_count(expected_expr, result, 1)
 
-    # alt and src for figure block should not be translated
-    expected_expr = """<img alt="img" src="_images/img.png" />"""
+    # src for figure block should not be translated (alt is translated)
+    expected_expr = """<img alt="IMG -&gt; I18N" src="_images/img.png" />"""
     assert_count(expected_expr, result, 1)
 
 
@@ -1138,8 +1146,13 @@ def test_additional_targets_should_be_translated(app):
     assert_count(expected_expr, result, 1)
 
     # C code block with lang should be translated and be *C* highlighted
-    expected_expr = ("""<span class="cp">#include</span> """
-                     """<span class="cpf">&lt;STDIO.H&gt;</span>""")
+    if pygments_version < (2, 10, 0):
+        expected_expr = ("""<span class="cp">#include</span> """
+                         """<span class="cpf">&lt;STDIO.H&gt;</span>""")
+    else:
+        expected_expr = ("""<span class="cp">#include</span>"""
+                         """<span class="w"> </span>"""
+                         """<span class="cpf">&lt;STDIO.H&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
     # literal block in list item should be translated
