@@ -11,6 +11,7 @@
 import sys
 from numbers import Integral
 from struct import Struct
+from types import TracebackType
 from typing import (Any, Callable, Dict, Generator, List, NewType, Optional, Tuple, TypeVar,
                     Union)
 
@@ -45,42 +46,58 @@ def test_restify():
     assert restify(None) == ":obj:`None`"
     assert restify(Integral) == ":class:`numbers.Integral`"
     assert restify(Struct) == ":class:`struct.Struct`"
-    assert restify(Any) == ":obj:`Any`"
+    assert restify(TracebackType) == ":class:`types.TracebackType`"
+    assert restify(Any) == ":obj:`~typing.Any`"
 
 
 def test_restify_type_hints_containers():
-    assert restify(List) == ":class:`List`"
-    assert restify(Dict) == ":class:`Dict`"
-    assert restify(List[int]) == ":class:`List`\\ [:class:`int`]"
-    assert restify(List[str]) == ":class:`List`\\ [:class:`str`]"
-    assert restify(Dict[str, float]) == ":class:`Dict`\\ [:class:`str`, :class:`float`]"
-    assert restify(Tuple[str, str, str]) == ":class:`Tuple`\\ [:class:`str`, :class:`str`, :class:`str`]"
-    assert restify(Tuple[str, ...]) == ":class:`Tuple`\\ [:class:`str`, ...]"
-    assert restify(List[Dict[str, Tuple]]) == ":class:`List`\\ [:class:`Dict`\\ [:class:`str`, :class:`Tuple`]]"
-    assert restify(MyList[Tuple[int, int]]) == ":class:`tests.test_util_typing.MyList`\\ [:class:`Tuple`\\ [:class:`int`, :class:`int`]]"
-    assert restify(Generator[None, None, None]) == ":class:`Generator`\\ [:obj:`None`, :obj:`None`, :obj:`None`]"
+    assert restify(List) == ":class:`~typing.List`"
+    assert restify(Dict) == ":class:`~typing.Dict`"
+    assert restify(List[int]) == ":class:`~typing.List`\\ [:class:`int`]"
+    assert restify(List[str]) == ":class:`~typing.List`\\ [:class:`str`]"
+    assert restify(Dict[str, float]) == (":class:`~typing.Dict`\\ "
+                                         "[:class:`str`, :class:`float`]")
+    assert restify(Tuple[str, str, str]) == (":class:`~typing.Tuple`\\ "
+                                             "[:class:`str`, :class:`str`, :class:`str`]")
+    assert restify(Tuple[str, ...]) == ":class:`~typing.Tuple`\\ [:class:`str`, ...]"
+    assert restify(Tuple[()]) == ":class:`~typing.Tuple`\\ [()]"
+    assert restify(List[Dict[str, Tuple]]) == (":class:`~typing.List`\\ "
+                                               "[:class:`~typing.Dict`\\ "
+                                               "[:class:`str`, :class:`~typing.Tuple`]]")
+    assert restify(MyList[Tuple[int, int]]) == (":class:`tests.test_util_typing.MyList`\\ "
+                                                "[:class:`~typing.Tuple`\\ "
+                                                "[:class:`int`, :class:`int`]]")
+    assert restify(Generator[None, None, None]) == (":class:`~typing.Generator`\\ "
+                                                    "[:obj:`None`, :obj:`None`, :obj:`None`]")
 
 
 def test_restify_type_hints_Callable():
-    assert restify(Callable) == ":class:`Callable`"
+    assert restify(Callable) == ":class:`~typing.Callable`"
 
     if sys.version_info >= (3, 7):
-        assert restify(Callable[[str], int]) == ":class:`Callable`\\ [[:class:`str`], :class:`int`]"
-        assert restify(Callable[..., int]) == ":class:`Callable`\\ [[...], :class:`int`]"
+        assert restify(Callable[[str], int]) == (":class:`~typing.Callable`\\ "
+                                                 "[[:class:`str`], :class:`int`]")
+        assert restify(Callable[..., int]) == (":class:`~typing.Callable`\\ "
+                                               "[[...], :class:`int`]")
     else:
-        assert restify(Callable[[str], int]) == ":class:`Callable`\\ [:class:`str`, :class:`int`]"
-        assert restify(Callable[..., int]) == ":class:`Callable`\\ [..., :class:`int`]"
+        assert restify(Callable[[str], int]) == (":class:`~typing.Callable`\\ "
+                                                 "[:class:`str`, :class:`int`]")
+        assert restify(Callable[..., int]) == (":class:`~typing.Callable`\\ "
+                                               "[..., :class:`int`]")
 
 
 def test_restify_type_hints_Union():
-    assert restify(Optional[int]) == ":obj:`Optional`\\ [:class:`int`]"
-    assert restify(Union[str, None]) == ":obj:`Optional`\\ [:class:`str`]"
-    assert restify(Union[int, str]) == ":obj:`Union`\\ [:class:`int`, :class:`str`]"
+    assert restify(Optional[int]) == ":obj:`~typing.Optional`\\ [:class:`int`]"
+    assert restify(Union[str, None]) == ":obj:`~typing.Optional`\\ [:class:`str`]"
+    assert restify(Union[int, str]) == ":obj:`~typing.Union`\\ [:class:`int`, :class:`str`]"
 
     if sys.version_info >= (3, 7):
-        assert restify(Union[int, Integral]) == ":obj:`Union`\\ [:class:`int`, :class:`numbers.Integral`]"
+        assert restify(Union[int, Integral]) == (":obj:`~typing.Union`\\ "
+                                                 "[:class:`int`, :class:`numbers.Integral`]")
         assert (restify(Union[MyClass1, MyClass2]) ==
-                ":obj:`Union`\\ [:class:`tests.test_util_typing.MyClass1`, :class:`tests.test_util_typing.<MyClass2>`]")
+                (":obj:`~typing.Union`\\ "
+                 "[:class:`tests.test_util_typing.MyClass1`, "
+                 ":class:`tests.test_util_typing.<MyClass2>`]"))
     else:
         assert restify(Union[int, Integral]) == ":class:`numbers.Integral`"
         assert restify(Union[MyClass1, MyClass2]) == ":class:`tests.test_util_typing.MyClass1`"
@@ -95,8 +112,12 @@ def test_restify_type_hints_typevars():
     assert restify(T) == ":obj:`tests.test_util_typing.T`"
     assert restify(T_co) == ":obj:`tests.test_util_typing.T_co`"
     assert restify(T_contra) == ":obj:`tests.test_util_typing.T_contra`"
-    assert restify(List[T]) == ":class:`List`\\ [:obj:`tests.test_util_typing.T`]"
-    assert restify(MyInt) == ":class:`MyInt`"
+    assert restify(List[T]) == ":class:`~typing.List`\\ [:obj:`tests.test_util_typing.T`]"
+
+    if sys.version_info >= (3, 10):
+        assert restify(MyInt) == ":class:`tests.test_util_typing.MyInt`"
+    else:
+        assert restify(MyInt) == ":class:`MyInt`"
 
 
 def test_restify_type_hints_custom_class():
@@ -108,7 +129,7 @@ def test_restify_type_hints_alias():
     MyStr = str
     MyTuple = Tuple[str, str]
     assert restify(MyStr) == ":class:`str`"
-    assert restify(MyTuple) == ":class:`Tuple`\\ [:class:`str`, :class:`str`]"  # type: ignore
+    assert restify(MyTuple) == ":class:`~typing.Tuple`\\ [:class:`str`, :class:`str`]"
 
 
 @pytest.mark.skipif(sys.version_info < (3, 7), reason='python 3.7+ is required.')
@@ -117,11 +138,25 @@ def test_restify_type_ForwardRef():
     assert restify(ForwardRef("myint")) == ":class:`myint`"
 
 
+@pytest.mark.skipif(sys.version_info < (3, 8), reason='python 3.8+ is required.')
+def test_restify_type_Literal():
+    from typing import Literal  # type: ignore
+    assert restify(Literal[1, "2", "\r"]) == ":obj:`~typing.Literal`\\ [1, '2', '\\r']"
+
+
+@pytest.mark.skipif(sys.version_info < (3, 9), reason='python 3.9+ is required.')
+def test_restify_pep_585():
+    assert restify(list[str]) == ":class:`list`\\ [:class:`str`]"  # type: ignore
+    assert restify(dict[str, str]) == ":class:`dict`\\ [:class:`str`, :class:`str`]"  # type: ignore
+    assert restify(dict[str, tuple[int, ...]]) == \
+        ":class:`dict`\\ [:class:`str`, :class:`tuple`\\ [:class:`int`, ...]]"  # type: ignore
+
+
 @pytest.mark.skipif(sys.version_info < (3, 10), reason='python 3.10+ is required.')
 def test_restify_type_union_operator():
-    assert restify(int | None) == "Optional[:class:`int`]"  # type: ignore
+    assert restify(int | None) == ":class:`int` | :obj:`None`"  # type: ignore
     assert restify(int | str) == ":class:`int` | :class:`str`"  # type: ignore
-    assert restify(int | str | None) == "Optional[:class:`int` | :class:`str`]"  # type: ignore
+    assert restify(int | str | None) == ":class:`int` | :class:`str` | :obj:`None`"  # type: ignore
 
 
 def test_restify_broken_type_hints():
@@ -133,7 +168,8 @@ def test_stringify():
     assert stringify(str) == "str"
     assert stringify(None) == "None"
     assert stringify(Integral) == "numbers.Integral"
-    assert restify(Struct) == ":class:`struct.Struct`"
+    assert stringify(Struct) == "struct.Struct"
+    assert stringify(TracebackType) == "types.TracebackType"
     assert stringify(Any) == "Any"
 
 
@@ -145,11 +181,24 @@ def test_stringify_type_hints_containers():
     assert stringify(Dict[str, float]) == "Dict[str, float]"
     assert stringify(Tuple[str, str, str]) == "Tuple[str, str, str]"
     assert stringify(Tuple[str, ...]) == "Tuple[str, ...]"
+    assert stringify(Tuple[()]) == "Tuple[()]"
     assert stringify(List[Dict[str, Tuple]]) == "List[Dict[str, Tuple]]"
     assert stringify(MyList[Tuple[int, int]]) == "tests.test_util_typing.MyList[Tuple[int, int]]"
     assert stringify(Generator[None, None, None]) == "Generator[None, None, None]"
 
 
+@pytest.mark.skipif(sys.version_info < (3, 9), reason='python 3.9+ is required.')
+def test_stringify_type_hints_pep_585():
+    assert stringify(list[int]) == "list[int]"
+    assert stringify(list[str]) == "list[str]"
+    assert stringify(dict[str, float]) == "dict[str, float]"
+    assert stringify(tuple[str, str, str]) == "tuple[str, str, str]"
+    assert stringify(tuple[str, ...]) == "tuple[str, ...]"
+    assert stringify(tuple[()]) == "tuple[()]"
+    assert stringify(list[dict[str, tuple]]) == "list[dict[str, tuple]]"
+    assert stringify(type[int]) == "type[int]"
+
+
 @pytest.mark.skipif(sys.version_info < (3, 9), reason='python 3.9+ is required.')
 def test_stringify_Annotated():
     from typing import Annotated  # type: ignore
@@ -194,11 +243,21 @@ def test_stringify_type_hints_typevars():
     T_co = TypeVar('T_co', covariant=True)
     T_contra = TypeVar('T_contra', contravariant=True)
 
-    assert stringify(T) == "T"
-    assert stringify(T_co) == "T_co"
-    assert stringify(T_contra) == "T_contra"
-    assert stringify(List[T]) == "List[T]"
-    assert stringify(MyInt) == "MyInt"
+    if sys.version_info < (3, 7):
+        assert stringify(T) == "T"
+        assert stringify(T_co) == "T_co"
+        assert stringify(T_contra) == "T_contra"
+        assert stringify(List[T]) == "List[T]"
+    else:
+        assert stringify(T) == "tests.test_util_typing.T"
+        assert stringify(T_co) == "tests.test_util_typing.T_co"
+        assert stringify(T_contra) == "tests.test_util_typing.T_contra"
+        assert stringify(List[T]) == "List[tests.test_util_typing.T]"
+
+    if sys.version_info >= (3, 10):
+        assert stringify(MyInt) == "tests.test_util_typing.MyInt"
+    else:
+        assert stringify(MyInt) == "MyInt"
 
 
 def test_stringify_type_hints_custom_class():
@@ -213,11 +272,17 @@ def test_stringify_type_hints_alias():
     assert stringify(MyTuple) == "Tuple[str, str]"  # type: ignore
 
 
+@pytest.mark.skipif(sys.version_info < (3, 8), reason='python 3.8+ is required.')
+def test_stringify_type_Literal():
+    from typing import Literal  # type: ignore
+    assert stringify(Literal[1, "2", "\r"]) == "Literal[1, '2', '\\r']"
+
+
 @pytest.mark.skipif(sys.version_info < (3, 10), reason='python 3.10+ is required.')
 def test_stringify_type_union_operator():
-    assert stringify(int | None) == "Optional[int]"  # type: ignore
+    assert stringify(int | None) == "int | None"  # type: ignore
     assert stringify(int | str) == "int | str"  # type: ignore
-    assert stringify(int | str | None) == "Optional[int | str]"  # type: ignore
+    assert stringify(int | str | None) == "int | str | None"  # type: ignore
 
 
 def test_stringify_broken_type_hints():
